------------------------------------------------------------
-- Undocumented DBCC 
------------------------------------------------------------

DBCC TRACEON (2588);
GO

DBCC HELP ('?');
GO

DBCC HELP ('autopilot'); -- What shall we try?
GO

/*
-- There are differences between the 2008 and 2008 R2 versions. 
-- The following commands exist in 2008/R2 but were gone in the next:
 
DBCC ACTIVECURSORS
DBCC DBRECOVER
DBCC ICECAPQUERY
DBCC LATCH
DBCC METADATA
DBCC MOVEPAGE
DBCC PRTIPAGE
DBCC SHOWDBAFFINITY
DBCC SHOWTABLEAFFINITY

-- SQL2012 brought three new commands:
 
DBCC CRASHDB
DBCC CSINDEX
DBCC readBytes

-- SQL2014 CTP2 (12.0.1524) delivered one more:
 
DBCC FLUSHQUERYSTORE
*/

------------------------------------------------------------
-- Optimizer Heuristics on/off 
------------------------------------------------------------

USE AdventureWorks2014;
GO

DBCC TRACEON (3604);

DBCC SHOWONRULES;
DBCC SHOWOFFRULES;

--Disable logical sort-merge or hash joins
DBCC RULEOFF('JNtoSM', 'JNtoHS');
DBCC RULEON('JNtoSM', 'JNtoHS');

DBCC FREEPROCCACHE; 

-- Ensure the four core implementation rules are available
DBCC RULEON ('GetToScan');
DBCC RULEON ('JNtoNL');
DBCC RULEON ('SelectToFilter');
DBCC RULEON ('GbAggToStrm')
 
-- Required by SQL Server 2005 only
-- DBCC RULEON ('ReduceGbAgg');

-- Alternative join implementations
DBCC RULEOFF ('JNtoHS');
DBCC RULEOFF ('JNtoSM');
 
-- Index-related transformations
DBCC RULEOFF ('GetIdxToRng');
DBCC RULEOFF ('GetToIdxScan');
DBCC RULEOFF ('SelIdxToRng');
DBCC RULEOFF ('SelToIdxStrategy');
DBCC RULEOFF ('SELonJN');
DBCC RULEOFF ('JNtoIdxLookup');
DBCC RULEOFF ('AppIdxToApp');
DBCC RULEOFF ('SelResToFilter');
DBCC RULEOFF ('WCJNonSELtoIdxLookup');
 
-- Exploration rules
DBCC RULEOFF ('GbAggToHS')
DBCC RULEOFF ('JoinCommute');
DBCC RULEOFF ('GbAggBeforeJoin');
DBCC RULEOFF ('GenLGAgg');
DBCC RULEOFF ('BuildSpool');
DBCC RULEOFF ('ImplRestrRemap');
DBCC RULEOFF ('EnforceSort');
DBCC RULEOFF ('NormalizeGbAgg');

DBCC FREEPROCCACHE; 

SELECT  P.ProductNumber, 
        P.ProductID, 
        total_qty = SUM(I.Quantity)
FROM    Production.Product P
JOIN    Production.ProductInventory I
        ON  I.ProductID = P.ProductID
WHERE   P.ProductNumber LIKE N'T%'
GROUP   BY
        P.ProductID,
        P.ProductNumber
OPTION  (RECOMPILE, QUERYTRACEON 8606);

DBCC RULEON ('JNtoHS');
DBCC RULEON ('JNtoSM');
DBCC RULEON ('BuildSpool');

--Clean up and re-enable
DBCC RULEON ('JNtoHS');
DBCC RULEON ('JNtoSM');
DBCC RULEON ('GetIdxToRng');
DBCC RULEON ('GetToIdxScan');
DBCC RULEON ('SelIdxToRng');
DBCC RULEON ('SelToIdxStrategy');
DBCC RULEON ('SELonJN');
DBCC RULEON ('JNtoIdxLookup');
DBCC RULEON ('AppIdxToApp');
DBCC RULEON ('SelResToFilter');
DBCC RULEON ('WCJNonSELtoIdxLookup');
DBCC RULEON ('GbAggToHS')
DBCC RULEON ('JoinCommute');
DBCC RULEON ('GbAggBeforeJoin');
DBCC RULEON ('GenLGAgg');
DBCC RULEON ('BuildSpool');
DBCC RULEON ('ImplRestrRemap');
DBCC RULEON ('EnforceSort');
DBCC RULEON ('NormalizeGbAgg');
DBCC RULEON ('ReduceGbAgg');

------------------------------------------------------------
-- Parsing And Binding 
------------------------------------------------------------
USE CorpDB;
GO

-- Turn on PARSEONLY.  This directs the query processor to only execute the "parse" phase.
SET PARSEONLY ON;

-- The following is a valid, executable query.  However, we get no results back because 
-- all SQL is doing is validating the syntax. Also note that we cannot get an execution plan.
SELECT * FROM CorpDB.dbo.Customer;
GO

-- The following is a valid SQL query but cannot be executed because the table doesn't exist.  
-- However, it will still run without error because it is syntactically correct.
SELECT NonExistantColumn FROM NonExistantTable;

SET PARSEONLY OFF;

-- A similar option is FMTONLY.  However, queries submitted will go through the parsing 
-- and binding process, but won't be fully optimized.
SET FMTONLY ON;

-- Now when we execute query it gets optimized, but it does not output any rows.
-- Still no execution plan.
SELECT *
FROM CorpDB.dbo.Customer c
	INNER JOIN CorpDB.dbo.OrderHeader oh on oh.CustomerId = c.CustomerID;

-- Cleanup
SET FMTONLY OFF;

-- Data type resolution:
-- Even though CustomerID and OrderId have completely different meanings, we can still UNION
-- the results together because they have the same data type (INT).
SELECT c.CustomerID
FROM CorpDB.dbo.Customer c
UNION
SELECT oh.OrderId
FROM CorpDB.dbo.OrderHeader oh;

-- However, FirstName (below) is of type VARCHAR() and OrderDate is DATETIME2. These types are
-- incompatible and we will get an error. Fails at binding step before optimization begins.
SELECT c.CustomerID, c.FirstName
FROM CorpDB.dbo.Customer c
UNION
SELECT oh.OrderId, oh.OrderDate
FROM CorpDB.dbo.OrderHeader oh;

-- Aggregate binding: 
-- The follow query will generate an error during the binding process because SQL knows that
-- FirstName is not a valid column because it is neither in the GROUP BY clause nor
-- is it an aggregate.

SELECT c.LastName, c.FirstName, COUNT(*) NbrCustomers
FROM CorpDB.dbo.Customer c
GROUP BY c.LastName;

------------------------------------------------------------
-- Trivial Plans 
------------------------------------------------------------

-- Get an estimated execution plan on this query.
-- Get the "F4" properties and select the root node.
-- Note that "Optimization Level" = TRIVIAL
SELECT *
FROM CorpDB.dbo.Customer c;

-- Same thing with a WHERE clause
SELECT *
FROM CorpDB.dbo.Customer c
WHERE c.State = 'NE';

-- Now let's add an index on State.
IF EXISTS (SELECT * FROM CorpDB.sys.indexes WHERE name = 'idx_Customer__State')
	DROP INDEX dbo.Customer.idx_Customer__State;

CREATE INDEX idx_Customer__State ON CorpDB.dbo.Customer (State);

-- Now get the estimated plan.
-- We still wind up with the same plan, but because SQL had alternative to consider this is no
-- longer a trivial plan.  "Optimization Level" = FULL.
SELECT *
FROM CorpDB.dbo.Customer c
WHERE c.State = 'NE'
OPTION (RECOMPILE);

-- Cleanup
IF EXISTS (SELECT * FROM CorpDB.sys.indexes WHERE name = 'idx_Customer__State')
	DROP INDEX dbo.Customer.idx_Customer__State;

-- Trivial plans may reference multiple tables. Why? Simplification!
WITH ALLDATA AS
(
	SELECT od.ProductId, p.ProductName, od.OrderId, od.Quantity, od.UnitPrice,
		oh.OrderDate, c.CustomerID, c.FirstName, c.LastName
	FROM CorpDB.dbo.Product p
	FULL JOIN CorpDB.dbo.OrderDetail od ON od.ProductId = p.ProductId
	FULL JOIN CorpDB.dbo.OrderHeader oh ON oh.OrderId = od.OrderId
	FULL JOIN CorpDB.dbo.Customer c ON c.CustomerID = oh.CustomerId
)
SELECT ad.OrderId, ad.Quantity, ad.UnitPrice
FROM AllData ad
WHERE ad.ProductId = 7304;

-- The simplification rules optimize away all but the OrderDetail table.
-- So the above query is essentially:
--     SELECT od.OrderId, od.Quantity, od.UnitPrice
--     FROM CorpDB.dbo.OrderDetail od
--     WHERE od.ProductId = 7304;

-- Note that we can prevent generating a trivial plan using TF8757.
SELECT *
FROM CorpDB.dbo.Customer c
OPTION (RECOMPILE, QUERYTRACEON 8757);

------------------------------------------------------------
-- Parse Trees and Some Memo Samples
------------------------------------------------------------

-- Turn on TF3604 for the session to ensure DBCC output appears
-- in the SSMS client the messages tab.
DBCC TRACEON (3604);

-- T8605 will output the "converted" parse tree.
-- Start with a simple parse tree.
SELECT c.CustomerID, c.FirstName, c.LastName, c.Address, c.City, c.State
FROM CorpDB.dbo.Customer c
OPTION (RECOMPILE, QUERYTRACEON 8605);

-- Query with a WHERE clause.
SELECT c.CustomerID, c.FirstName, c.LastName, c.Address, c.City, c.State
FROM CorpDB.dbo.Customer c
WHERE c.State = 'NE'
OPTION (RECOMPILE, QUERYTRACEON 8605);

-- Query with a WHERE clause (LIKE).
SELECT c.CustomerID, c.FirstName, c.LastName, c.Address, c.City, c.State
FROM CorpDB.dbo.Customer c
WHERE c.LastName LIKE 'Smi%'
OPTION (RECOMPILE, QUERYTRACEON 8605);

-- Query with a JOIN.
SELECT c.CustomerID, c.FirstName, c.LastName, c.Address, c.City, c.State, oh.OrderId, oh.OrderDate
FROM CorpDB.dbo.OrderHeader oh
   INNER JOIN CorpDB.dbo.Customer c ON c.CustomerID = oh.CustomerId
OPTION (RECOMPILE, QUERYTRACEON 8605);

-- Query with an ORDER BY.
SELECT c.CustomerID, c.FirstName, c.LastName, c.Address, c.City, c.State, oh.OrderId, oh.OrderDate
FROM CorpDB.dbo.OrderHeader oh
	INNER JOIN CorpDB.dbo.Customer c ON c.CustomerID = oh.CustomerId
ORDER BY c.City, c.State, c.LastName, c.FirstName
OPTION (RECOMPILE, QUERYTRACEON 8605);

-- Query with a GROUP BY.
SELECT c.CustomerID, COUNT(oh.OrderId) NbrOrders
FROM CorpDB.dbo.OrderHeader oh
	INNER JOIN CorpDB.dbo.Customer c ON c.CustomerID = oh.CustomerId
GROUP BY c.CustomerID
OPTION (RECOMPILE, QUERYTRACEON 8605);

-- Query with a DISTINCT.
SELECT distinct c.FirstName, c.LastName
FROM CorpDB.dbo.Customer c
WHERE c.State = 'NE'
OPTION (RECOMPILE, QUERYTRACEON 8605);

-- Query with a UNION.
SELECT p.ProductId
FROM CorpDB.dbo.Product p
UNION ALL
SELECT od.ProductId
FROM CorpDB.dbo.OrderDetail od
OPTION (RECOMPILE, QUERYTRACEON 8605);

SELECT p.ProductId
FROM CorpDB.dbo.Product p
UNION
SELECT od.ProductId
FROM CorpDB.dbo.OrderDetail od
OPTION (RECOMPILE, QUERYTRACEON 8605);

-- Query with an EXCEPT.
SELECT od.ProductId
FROM CorpDB.dbo.OrderDetail od
EXCEPT
SELECT p.ProductId
FROM CorpDB.dbo.Product p
OPTION (RECOMPILE, QUERYTRACEON 8605);

-- TF8606 will output the parse tree at various stages of optimization.
-- Trees outputted are: Input, Simplified, Join-Collapsed, Before Project Normalization, 
--    After Project Normalization
-- The following query has a GROUP BY.
SELECT c.CustomerID, count(oh.OrderId) NbrOrders
FROM CorpDB.dbo.OrderHeader oh
	INNER JOIN CorpDB.dbo.Customer c ON c.CustomerID = oh.CustomerId
GROUP BY c.CustomerID
OPTION (RECOMPILE, QUERYTRACEON 8606);

-- Can combine TF8605 and TF8606 for a more complete picture.
SELECT c.CustomerID, count(oh.OrderId) NbrOrders
FROM CorpDB.dbo.OrderHeader oh
	INNER JOIN CorpDB.dbo.Customer c ON c.CustomerID = oh.CustomerId
GROUP BY c.CustomerID
OPTION (RECOMPILE, QUERYTRACEON 8605, QUERYTRACEON 8606);

-- Further combine with TF8621 to see the specific optimizer rules applied.
SELECT c.CustomerID, COUNT(oh.OrderId) NbrOrders
FROM CorpDB.dbo.OrderHeader oh
	INNER JOIN CorpDB.dbo.Customer c ON c.CustomerID = oh.CustomerId
GROUP BY c.CustomerID
OPTION (RECOMPILE, QUERYTRACEON 8605, QUERYTRACEON 8606, QUERYTRACEON 8621);

-- TF8607 shows the "output memo tree".  This will include PHYSICAL operators.
SELECT c.CustomerID, count(oh.OrderId) NbrOrders
FROM CorpDB.dbo.OrderHeader oh
	INNER JOIN CorpDB.dbo.Customer c on c.CustomerID = oh.CustomerId
GROUP BY c.CustomerID
OPTION (RECOMPILE, QUERYTRACEON 8607);

-- TF7352 shows another version of the final memo tree.
SELECT c.CustomerID, count(oh.OrderId) NbrOrders
FROM CorpDB.dbo.OrderHeader oh
	INNER JOIN CorpDB.dbo.Customer c on c.CustomerID = oh.CustomerId
GROUP BY c.CustomerID
OPTION (RECOMPILE, QUERYTRACEON 7352);

-- Query that shows join-collapse
SELECT oh.OrderId, oh.OrderDate, c.CustomerID, c.FirstName, c.LastName,
	od.OrderId, od.Quantity, od.UnitPrice
FROM CorpDB.dbo.OrderHeader oh
	INNER JOIN CorpDB.dbo.Customer c on c.CustomerID = oh.CustomerId
	INNER JOIN CorpDB.dbo.OrderDetail od on od.OrderId = oh.OrderId
OPTION (RECOMPILE, QUERYTRACEON 8605, QUERYTRACEON 8606);

-------------------------------------------------------------------------------
-- Simplification
-------------------------------------------------------------------------------

-- Get estimated execution plan on these two queries. Note that the optimizer makes 
-- the same plan for them because the subquery in the first is re-written as an 
-- INNER JOIN (as in query 2).
SELECT oh.OrderId, oh.OrderDate, oh.CustomerId
FROM CorpDB.dbo.OrderHeader oh
WHERE oh.CustomerId IN
	(
		SELECT c.CustomerId
		FROM CorpDB.dbo.Customer c
		WHERE c.State = 'NE'
	)
OPTION (RECOMPILE, QUERYTRACEON 8605, QUERYTRACEON 8606);

-- Another subquery to INNER JOIN
-- Note how the "simplified" tree treats the query as if it had been written without 
-- the subquery.
-- MAXDOP 1 specified to simplify the graphical query plan, but works without MAXDOP 
-- as well.
SELECT CustomerOrderView.OrderId, CustomerOrderView.CustomerId, od.ProductId
FROM CorpDB.dbo.OrderDetail od
	INNER JOIN
	(
		SELECT oh.OrderId, oh.CustomerId, c.State
		FROM CorpDB.dbo.OrderHeader oh
		INNER JOIN CorpDB.dbo.Customer c on oh.CustomerId = c.CustomerId
	) CustomerOrderView on od.OrderId = CustomerOrderView.OrderId
WHERE CustomerOrderView.State = 'NE'
OPTION (MAXDOP 1, RECOMPILE, QUERYTRACEON 8605, QUERYTRACEON 8606);

-- Predicate pushdown
-- Note that the select logical operation (LogOp_Select) is pushed below the join
SELECT oh.OrderId, oh.OrderDate, oh.CustomerId
FROM CorpDB.dbo.OrderHeader oh
	INNER JOIN CorpDB.dbo.Customer c on oh.CustomerId = c.CustomerID
WHERE c.State = 'NE'
OPTION (RECOMPILE, QUERYTRACEON 8605, QUERYTRACEON 8606);

-- Contradiction detection: 
-- Get an estimated plan on this query.
-- Note that we need to include a JOIN in the query to avoid getting a trivial plan.
-- SQL Server recognizes that the WHERE conditions are contradictory and removes all 
-- data access.
SELECT p.ProductId, p.ProductName, p.UnitPrice
FROM CorpDB.dbo.Product p
	INNER JOIN CorpDB.dbo.OrderDetail od on od.ProductId = p.ProductId
WHERE p.UnitPrice > 50.00
	AND p.UnitPrice < 25.00;
GO

-- Foreign key table removal:
-- Get an estimated query plan on this query. Note that SQL removes the Customer 
-- table entirely from the plan because the FK guarantees that the row exists in 
-- Customer, and no columns are otherwise required from Customer.
SELECT oh.OrderId, oh.OrderDate, oh.CustomerId
FROM CorpDB.dbo.OrderHeader oh
	INNER JOIN CorpDB.dbo.Customer c on oh.CustomerId = c.CustomerID;

-- We temporarily disable the FK.
ALTER TABLE CorpDB.dbo.OrderHeader NOCHECK CONSTRAINT fk_OrderHeader__Customer;

-- Get the estimated plan again. Now it MUST actually access the Customer table.
SELECT oh.OrderId, oh.OrderDate, oh.CustomerId
FROM CorpDB.dbo.OrderHeader oh
	INNER JOIN CorpDB.dbo.Customer c on oh.CustomerId = c.CustomerID;

-- Re-enable the FK.
ALTER TABLE CorpDB.dbo.OrderHeader CHECK CONSTRAINT fk_OrderHeader__Customer;

-- Get the estimated plan again.
-- SQL Server knows that there may have been changes to OrderHeader that don't 
-- comply with the FK, so Customer will still be accessed.
SELECT oh.OrderId, oh.OrderDate, oh.CustomerId
FROM CorpDB.dbo.OrderHeader oh
	INNER JOIN CorpDB.dbo.Customer c on oh.CustomerId = c.CustomerID;

-- Enable the FK and validate the data.
ALTER TABLE CorpDB.dbo.OrderHeader WITH CHECK CHECK CONSTRAINT fk_OrderHeader__Customer;

-- Get the estimated plan again. As before, SQL Server is now satisfied of data 
-- integrity and once again skip physical access to Customer table.
SELECT oh.OrderId, oh.OrderDate, oh.CustomerId
FROM CorpDB.dbo.OrderHeader oh
	INNER JOIN CorpDB.dbo.Customer c on oh.CustomerId = c.CustomerID;

-- Bushy Trees
-- Normally, the optimizer prevents bushy tree execution plans. But poor coding
-- practices can cause them. 

-- Get an estimated execution plan on the following two queries.   
-- Note query 1 that the plan joins a pair of tables, pulls in another
-- table, then the final table.
SELECT *
FROM CorpDB.dbo.OrderHeader oh
	JOIN CorpDB.dbo.OrderDetail od ON oh.OrderId = od.OrderId
	JOIN CorpDB.dbo.Customer c ON oh.CustomerId = c.CustomerID
	JOIN CorpDB.dbo.Product p ON od.ProductId = p.ProductId;

-- Otoh, our code can also force a bushy plan to happen.
-- Comparitively, it's about twice as expensive as the optimized query.
SELECT *
FROM CorpDB.dbo.Customer c
	JOIN CorpDB.dbo.OrderHeader oh ON oh.CustomerId = c.CustomerID
		JOIN (CorpDB.dbo.Product p
			JOIN CorpDB.dbo.OrderDetail od ON od.ProductId = p.ProductId) ON oh.OrderId = od.OrderId
OPTION (FORCE ORDER);
GO

-------------------------------------------------------------------------------
-- Aggregate Pushdown
-------------------------------------------------------------------------------

-- Get an estimated query plan for the following queries as a batch and compare.
-- This is a traditional aggregation where the optimizer first joins our two tables, 
-- then aggregates the result.
WITH CustomerCount AS
(
	SELECT oh.CustomerId, COUNT(*) NbrOrders
	FROM CorpDB.dbo.OrderHeader oh
	GROUP BY oh.CustomerId
)
SELECT c.CustomerID, c.FirstName, c.LastName, cc.NbrOrders
FROM CorpDB.dbo.Customer c
	INNER JOIN CustomerCount cc ON c.CustomerID = cc.CustomerId
WHERE c.State = 'NE';

-- With a large input from Customer (no predicate), the optimizer decides to first 
-- aggregate on OrderHeader, then to join the aggregate results to Customer.
WITH CustomerCount AS
(
	SELECT oh.CustomerId, count(*) NbrOrders
	FROM CorpDB.dbo.OrderHeader oh
	GROUP BY oh.CustomerId
)
SELECT c.CustomerID, c.FirstName, c.LastName, cc.NbrOrders
FROM CorpDB.dbo.Customer c
	INNER JOIN CustomerCount cc ON c.CustomerID = cc.CustomerId;

-- Now the table order is reversed.
WITH CustomerCount AS
(
	SELECT oh.CustomerId, COUNT(*) NbrOrders
	FROM CorpDB.dbo.OrderHeader oh
	GROUP BY oh.CustomerId
)
SELECT c.CustomerID, c.FirstName, c.LastName, cc.NbrOrders
FROM CorpDB.dbo.Customer c
	INNER JOIN CustomerCount cc ON c.CustomerID = cc.CustomerId
WHERE c.State = 'CA';

-------------------------------------------------------------------------------
-- Memo Structures 
-------------------------------------------------------------------------------

DBCC TRACEON (3604);

/*
From Paul White, http://sqlblog.com/blogs/paul_white/archive/2012/04/29/query-optimizer-deep-dive-part-3.aspx

Each group in the Memo initially contains just one entry  a node from the input logical tree.  
As exploration and implementation phases progress, new groups may be added, and new physical and 
logical alternatives may be added to existing groups (all alternatives in a group share the same 
logical properties, but will often have quite different physical properties).  Groups are also shared 
between plan alternatives where possible, allowing the search of more plans in the same time and 
space.

Below, the input tree to cost-based optimization is first copied into the Memo structure.
*/
USE AdventureWorks2014;
GO

SET NOCOUNT ON;
DBCC FREEPROCCACHE;
DBCC TRACEON(3604);
GO

-- Enable graphic execution plan and assess.
SELECT c.customerid, COUNT(*)
FROM sales.customer AS c
   JOIN sales.SalesOrderHeader AS s 
      ON c.customerid = s.customerid
WHERE c.territoryid = 4
GROUP BY c.customerid;

-- Now, enable trace flag 8605 to see the initial logical parse tree. Compare.
SELECT productid, name
FROM production.product
WHERE productid = 877
OPTION (RECOMPILE, QUERYTRACEON 8605);

-- It gets verbose very quickly!
SELECT c.customerid, COUNT(*)
FROM sales.customer AS c
	JOIN sales.SalesOrderHeader AS s ON c.customerid = s.customerid
WHERE c.territoryid = 4
GROUP BY c.customerid
OPTION (RECOMPILE, QUERYTRACEON 8605);

-- Trace flag 8606 shows the tree at various stages
SELECT p.Name, Total = SUM(inv.Quantity)
FROM Production.Product AS p
   JOIN Production.ProductInventory AS inv 
      ON inv.ProductID = p.ProductID
WHERE p.Name LIKE N'[A-G]%'
GROUP BY p.Name
OPTION (QUERYTRACEON 8606);

-- Initial memo contents
SELECT p.Name, Total = SUM(inv.Quantity)
FROM Production.Product AS p
   JOIN Production.ProductInventory AS inv 
      ON inv.ProductID = p.ProductID
WHERE p.Name LIKE N'[A-G]%'
GROUP BY p.Name
OPTION (RECOMPILE, QUERYTRACEON 8608);

-- Final memo
SELECT p.Name, Total = SUM(inv.Quantity)
FROM Production.Product AS p
   JOIN Production.ProductInventory AS inv 
      ON inv.ProductID = p.ProductID
WHERE p.Name LIKE N'[A-G]%'
GROUP BY p.Name
OPTION (RECOMPILE, QUERYTRACEON 8615);

-- Rather than the details of T8615, let's summarize each search phase using T8576.
-- Note that the query might run Search 1 twice (once for serial, once for parallel)
SELECT p.Name, Total = SUM(inv.Quantity)
FROM Production.Product AS p
   JOIN Production.ProductInventory AS inv 
      ON inv.ProductID = p.ProductID
WHERE p.Name LIKE N'[A-G]%'
GROUP BY p.Name
OPTION (RECOMPILE, QUERYTRACEON 8675);

-- Again, here's a more verbose summary of operations via T2372.
-- Search 0, 1, and 2 are called "TP", "QuickPlan", and "Full" using this trace flag.
SELECT p.Name, Total = SUM(inv.Quantity)
FROM Production.Product AS p
   JOIN Production.ProductInventory AS inv 
      ON inv.ProductID = p.ProductID
WHERE p.Name LIKE N'[A-G]%'
GROUP BY p.Name
OPTION (RECOMPILE, QUERYTRACEON 2372);

-- "None of this makes sense, help me out here!" 
-- Then, use T8619 for more rule description info.
SELECT p.Name, Total = SUM(inv.Quantity)
FROM Production.Product AS p
   JOIN Production.ProductInventory AS inv 
      ON inv.ProductID = p.ProductID
WHERE p.Name LIKE N'[A-G]%'
GROUP BY p.Name
OPTION (RECOMPILE, QUERYTRACEON 8619);

-- On the other hand, maybe I get it. Give me more explicit and verbose information.
SELECT p.Name, Total = SUM(inv.Quantity)
FROM Production.Product AS p
   JOIN Production.ProductInventory AS inv 
      ON inv.ProductID = p.ProductID
WHERE p.Name LIKE N'[A-G]%'
GROUP BY p.Name
OPTION (RECOMPILE, QUERYTRACEON 8615
,QUERYTRACEON 8609 -- <-- Optimization Stats
,QUERYTRACEON 8739 -- <-- Group Optimization Info
); 

-- Statistics Internal Metadata:
-- Use trace flags 9292 and 9204 to show information about the statistics loaded 
-- during the optimization process.
SELECT TOP 5 od.ProductId, SUM(od.Quantity) - 20 ExcessOrders
FROM CorpDB.dbo.OrderHeader oh
	INNER JOIN CorpDB.dbo.OrderDetail od 
		ON oh.OrderId = od.OrderId
	INNER JOIN CorpDB.dbo.Customer cust 
		ON oh.CustomerId = cust.CustomerID
WHERE cust.State = 'NE'
GROUP BY od.ProductId
	HAVING SUM(od.Quantity) >= 20
ORDER BY od.ProductId
OPTION (RECOMPILE, QUERYTRACEON 9292, QUERYTRACEON 9204);

-------------------------------------------------------------------------------
-- Give the Optimizer More Time ###
-------------------------------------------------------------------------------

-- Give the optimizer more "time" with -TF8780. Also, note -T8671.
USE AdventureWorks2014;
GO

PRINT '
    ****
	Base query, showing optimization phases via -T8675
	****';
GO 

SET STATISTICS TIME ON;
GO

SELECT *
FROM Sales.SalesOrderDetail sod
    JOIN Sales.SalesOrderHeader soh ON sod.SalesOrderID = soh.SalesOrderID
    JOIN Sales.CreditCard cc ON cc.CreditCardID = soh.CreditCardID
    JOIN Sales.CurrencyRate cr ON cr.CurrencyRateID = soh.CurrencyRateID
    LEFT JOIN Sales.Currency crn ON crn.CurrencyCode = cr.FromCurrencyCode AND crn.ModifiedDate = cr.ModifiedDate
    JOIN Sales.SalesPerson sp ON soh.SalesPersonID = sp.BusinessEntityID
    LEFT JOIN Sales.SalesTerritory st ON soh.TerritoryID = st.TerritoryID
    JOIN Person.BusinessEntity be ON sp.BusinessEntityID = be.BusinessEntityID
    JOIN Person.Person p ON p.BusinessEntityID = be.BusinessEntityID
    JOIN Person.BusinessEntityAddress bea ON bea.BusinessEntityID = be.BusinessEntityID
    JOIN Person.Address a ON bea.AddressID = a.AddressID
    LEFT JOIN Person.StateProvince spr ON a.StateProvinceID = spr.StateProvinceID
    JOIN (
        SELECT MAX(tha.TransactionDate) AS m, tha.ProductID 
		FROM Production.TransactionHistoryArchive AS tha 
		GROUP BY tha.ProductID
    ) tha ON tha.ProductID = sod.ProductID
WHERE
    cc.CardType IN ('SuperiorCard','Vista') AND
    p.PersonType = 'SP' AND
    (soh.SalesOrderNumber >= 'SO' OR spr.CountryRegionCode IN ('FR', 'DE', 'GB'))
OPTION (RECOMPILE, QUERYTRACEON 3604, QUERYTRACEON 8675);
GO

SET STATISTICS TIME OFF;
GO

PRINT '
    ****
	With -T8780 added
	****';
GO 

SET STATISTICS TIME ON;
GO

SELECT *
FROM Sales.SalesOrderDetail sod
    JOIN Sales.SalesOrderHeader soh ON sod.SalesOrderID = soh.SalesOrderID
    JOIN Sales.CreditCard cc ON cc.CreditCardID = soh.CreditCardID
    JOIN Sales.CurrencyRate cr ON cr.CurrencyRateID = soh.CurrencyRateID
    LEFT JOIN Sales.Currency crn ON crn.CurrencyCode = cr.FromCurrencyCode AND crn.ModifiedDate = cr.ModifiedDate
    JOIN Sales.SalesPerson sp ON soh.SalesPersonID = sp.BusinessEntityID
    LEFT JOIN Sales.SalesTerritory st ON soh.TerritoryID = st.TerritoryID
    JOIN Person.BusinessEntity be ON sp.BusinessEntityID = be.BusinessEntityID
    JOIN Person.Person p ON p.BusinessEntityID = be.BusinessEntityID
    JOIN Person.BusinessEntityAddress bea ON bea.BusinessEntityID = be.BusinessEntityID
    JOIN Person.Address a ON bea.AddressID = a.AddressID
    LEFT JOIN Person.StateProvince spr ON a.StateProvinceID = spr.StateProvinceID
    JOIN (
        SELECT MAX(tha.TransactionDate) AS m, tha.ProductID 
		FROM Production.TransactionHistoryArchive AS tha 
		GROUP BY tha.ProductID
    ) tha ON tha.ProductID = sod.ProductID
WHERE
    cc.CardType IN ('SuperiorCard','Vista') AND
    p.PersonType = 'SP' AND
    (soh.SalesOrderNumber >= 'SO' OR spr.CountryRegionCode IN ('FR', 'DE', 'GB'))
OPTION (RECOMPILE, QUERYTRACEON 3604, QUERYTRACEON 8675, QUERYTRACEON 8780);
go

SET STATISTICS TIME OFF;
GO

PRINT '
    ****
	With -T8780 and -T8681 added
	****';
GO 

SET STATISTICS TIME ON;
GO

SELECT *
FROM Sales.SalesOrderDetail sod
    JOIN Sales.SalesOrderHeader soh ON sod.SalesOrderID = soh.SalesOrderID
    JOIN Sales.CreditCard cc ON cc.CreditCardID = soh.CreditCardID
    JOIN Sales.CurrencyRate cr ON cr.CurrencyRateID = soh.CurrencyRateID
    LEFT JOIN Sales.Currency crn ON crn.CurrencyCode = cr.FromCurrencyCode AND crn.ModifiedDate = cr.ModifiedDate
    JOIN Sales.SalesPerson sp ON soh.SalesPersonID = sp.BusinessEntityID
    LEFT JOIN Sales.SalesTerritory st ON soh.TerritoryID = st.TerritoryID
    JOIN Person.BusinessEntity be ON sp.BusinessEntityID = be.BusinessEntityID
    JOIN Person.Person p ON p.BusinessEntityID = be.BusinessEntityID
    JOIN Person.BusinessEntityAddress bea ON bea.BusinessEntityID = be.BusinessEntityID
    JOIN Person.Address a ON bea.AddressID = a.AddressID
    LEFT JOIN Person.StateProvince spr ON a.StateProvinceID = spr.StateProvinceID
    JOIN (
        SELECT MAX(tha.TransactionDate) AS m, tha.ProductID 
		FROM Production.TransactionHistoryArchive AS tha 
		GROUP BY tha.ProductID
    ) tha ON tha.ProductID = sod.ProductID
WHERE
    cc.CardType IN ('SuperiorCard','Vista') AND
    p.PersonType = 'SP' AND
    (soh.SalesOrderNumber >= 'SO' OR spr.CountryRegionCode IN ('FR', 'DE', 'GB'))
OPTION (RECOMPILE, QUERYTRACEON 3604, QUERYTRACEON 8675, QUERYTRACEON 8780, QUERYTRACEON 8671);
GO

SET STATISTICS TIME OFF;
GO

