-- Query Telemetry

-- Show rollups of the kind of optimizations happening on this instance.
SELECT * 
FROM sys.dm_exec_query_optimizer_info;

-- An example of using the DMV to show types of specific optimizer activities.
SELECT
	(SELECT occurrence 
	 FROM sys.dm_exec_query_optimizer_info
	 WHERE counter = 'hints') * 100 / 
	(SELECT occurrence 
	 FROM sys.dm_exec_query_optimizer_info
	 WHERE counter = 'optimizations') AS 'percent_hints';

-- To see the names used by SQL Server logical parse trees.
SELECT *
FROM sys.dm_xe_map_values
WHERE name = 'query_optimizer_tree_id';

-- See in what transformations the query optimizer is applying since last 
-- restart of the instance.
SELECT *
FROM sys.dm_exec_query_transformation_stats;

-- sys.dm_exec_query_transformation_stats shows one record for each transformation rule 
-- that the optimizer has. It includes some "promise" information that is a guess about 
-- how effective the rule will be under the current circumstances (which allows the 
-- optimizer to prioritize rules).  It also contains columns indicating how successful 
-- the transformation was.

-- Let's get pertinent information, say cardinality estimates, via XEvents.
CREATE EVENT SESSION [XeNewCE] ON SERVER
ADD EVENT sqlserver.query_optimizer_estimate_cardinality(
    ACTION(sqlserver.sql_text)),
ADD EVENT sqlserver.query_optimizer_force_both_cardinality_estimation_behaviors
ADD TARGET package0.event_file(SET filename=N'C:\Perflogs\XeNewCE.xel',max_file_size=(50),max_rollover_files=(2))
GO

ALTER EVENT SESSION [XeNewCE] ON SERVER STATE = START
GO

SELECT TOP 5 od.ProductId, SUM(od.Quantity) - 20 ExcessOrders
FROM CorpDB.dbo.OrderHeader oh
	INNER JOIN CorpDB.dbo.OrderDetail od ON oh.OrderId = od.OrderId
	INNER JOIN CorpDB.dbo.Customer cust ON oh.CustomerId = cust.CustomerID
WHERE cust.State = 'NE'
GROUP BY od.ProductId
	HAVING SUM(od.Quantity) >= 20
ORDER BY od.ProductId

-- Now check the SSMS XEvents interface, Object Explorer >> Management >> Extended Events

ALTER EVENT SESSION [XeNewCE] ON SERVER STATE = STOP;
GO

DROP EVENT SESSION [XeNewCE] ON SERVER;
GO

/*
What statistics are being used for a given query?

Then you can look at the XE session and look at the information there, such as:
-- Calculator is the algorithm used to estimate cardinality
-- -- Subcalculator shows predicate cardinality
-- Input_relation - shows input tree
-- StatsCollection will show the loaded statistics.
-- -- StatsCollectionID appears as a property of the operator in the plan as well.
-- -- The StatsCollectionID is in the Calculator which tells you what stats is used for what predicate.
*/

------------------------------------------------------------
-- Before and after snapshots of important DMVs
------------------------------------------------------------

-- Snapshot example for sys.dm_exec_query_optimizer_info. Take before- and 
-- of the DMV when executing a query to see optimizer info for that specific 
-- query. Note that this is not a surefire way to filter out other SQL optimizer
-- activity on the instance. Like all DMVs, the stats are accumulated since the 
-- instance start.

-- sys.dm_exec_query_transformation_stats shows one record for each transformation rule 
-- that the optimizer has. It includes some "promise" information that is a guess about 
-- how effective the rule will be under the current circumstances (which allows the 
-- optimizer to prioritize rules).  It also contains columns indicating how successful 
-- the transformation was.

SELECT * INTO #before_optimizer_info FROM sys.dm_exec_query_optimizer_info;
GO
SELECT * INTO #after_optimizer_info FROM sys.dm_exec_query_optimizer_info;
GO
IF OBJECT_ID('tempdb..#before_optimizer_info') IS NOT NULL DROP TABLE #before_optimizer_info;
IF OBJECT_ID('tempdb..#after_optimizer_info') IS NOT NULL DROP TABLE #after_optimizer_info;
GO
SELECT * INTO #before_optimizer_info FROM sys.dm_exec_query_optimizer_info;
GO

-------------------------------------------------------------------------------
-- Insert query of interest here
-------------------------------------------------------------------------------

SELECT * INTO #after_optimizer_info FROM sys.dm_exec_query_optimizer_info;
GO
SELECT a.counter, a.occurrence - b.occurrence occurrence, a.occurrence * a.value - b.occurrence * b.value value
FROM #before_optimizer_info b
	JOIN #after_optimizer_info a on a.counter = b.counter
WHERE a.occurrence != b.occurrence ORDER BY counter;
GO
IF OBJECT_ID('tempdb..#before_optimizer_info') IS NOT NULL DROP TABLE #before_optimizer_info;
IF OBJECT_ID('tempdb..#after_optimizer_info') IS NOT NULL DROP TABLE #after_optimizer_info;
go
-------------------------------------------------------------------------------
-- End
-------------------------------------------------------------------------------




-------------------------------------------------------------------------------
-- Begin example by turning on graphic execution plans in SSMS
-------------------------------------------------------------------------------

SELECT * INTO #before_optimizer_info FROM sys.dm_exec_query_optimizer_info;
GO
SELECT * INTO #after_optimizer_info FROM sys.dm_exec_query_optimizer_info;
GO
IF OBJECT_ID('tempdb..#before_optimizer_info') IS NOT NULL DROP TABLE #before_optimizer_info;
IF OBJECT_ID('tempdb..#after_optimizer_info') IS NOT NULL DROP TABLE #after_optimizer_info;
GO
SELECT * INTO #before_optimizer_info FROM sys.dm_exec_query_optimizer_info;
GO
-------------------------------------------------------------------------------
-- Insert query of interest here
-------------------------------------------------------------------------------
SELECT TOP 5 od.ProductId, SUM(od.Quantity) - 20 ExcessOrders
FROM CorpDB.dbo.OrderHeader oh
	INNER JOIN CorpDB.dbo.OrderDetail od ON oh.OrderId = od.OrderId
	INNER JOIN CorpDB.dbo.Customer cust ON oh.CustomerId = cust.CustomerID
WHERE cust.State = 'NE'
GROUP BY od.ProductId
	HAVING SUM(od.Quantity) >= 20
ORDER BY od.ProductId;
GO

SELECT * INTO #after_optimizer_info FROM sys.dm_exec_query_optimizer_info;
GO
SELECT a.counter, a.occurrence - b.occurrence occurrence, a.occurrence * a.value - b.occurrence * b.value value
FROM #before_optimizer_info b
	JOIN #after_optimizer_info a ON a.counter = b.counter
WHERE a.occurrence != b.occurrence ORDER BY counter;
GO
IF OBJECT_ID('tempdb..#before_optimizer_info') IS NOT NULL DROP TABLE #before_optimizer_info;
IF OBJECT_ID('tempdb..#after_optimizer_info') IS NOT NULL DROP TABLE #after_optimizer_info;
GO
-------------------------------------------------------------------------------
-- End
-------------------------------------------------------------------------------

-- Interesting to observe how CTEs are handled by the optimizer.
-- First, without a CTE.  Note no change in view references.

-------------------------------------------------------------------------------
-- Begin
-------------------------------------------------------------------------------

SELECT * INTO #before_optimizer_info FROM sys.dm_exec_query_optimizer_info;
GO
SELECT * INTO #after_optimizer_info FROM sys.dm_exec_query_optimizer_info;
GO
IF OBJECT_ID('tempdb..#before_optimizer_info') IS NOT NULL DROP TABLE #before_optimizer_info;
IF OBJECT_ID('tempdb..#after_optimizer_info') IS NOT NULL DROP TABLE #after_optimizer_info;
GO
SELECT * INTO #before_optimizer_info FROM sys.dm_exec_query_optimizer_info;
GO
-------------------------------------------------------------------------------
-- Insert query of interest here
-------------------------------------------------------------------------------
SELECT oh.OrderId, oh.OrderDate, c.FirstName, c.LastName
FROM CorpDB.dbo.OrderHeader oh
	INNER JOIN CorpDB.dbo.Customer c ON oh.CustomerId = c.CustomerID
WHERE c.State = 'NE'
OPTION (RECOMPILE);
GO

SELECT * INTO #after_optimizer_info FROM sys.dm_exec_query_optimizer_info;
GO
SELECT a.counter, a.occurrence - b.occurrence occurrence, a.occurrence * a.value - b.occurrence * b.value value
FROM #before_optimizer_info b
	JOIN #after_optimizer_info a ON a.counter = b.counter
WHERE a.occurrence != b.occurrence ORDER BY counter;
GO
IF OBJECT_ID('tempdb..#before_optimizer_info') IS NOT NULL DROP TABLE #before_optimizer_info;
IF OBJECT_ID('tempdb..#after_optimizer_info') IS NOT NULL DROP TABLE #after_optimizer_info;
GO
-------------------------------------------------------------------------------
-- End
-------------------------------------------------------------------------------

-- Now with a CTE. Note that "view reference" is bumped by 1.

-------------------------------------------------------------------------------
-- Begin
-------------------------------------------------------------------------------

SELECT * INTO #before_optimizer_info FROM sys.dm_exec_query_optimizer_info;
GO
SELECT * INTO #after_optimizer_info FROM sys.dm_exec_query_optimizer_info;
GO
IF OBJECT_ID('tempdb..#before_optimizer_info') IS NOT NULL DROP TABLE #before_optimizer_info;
IF OBJECT_ID('tempdb..#after_optimizer_info') IS NOT NULL DROP TABLE #after_optimizer_info;
GO
SELECT * INTO #before_optimizer_info FROM sys.dm_exec_query_optimizer_info;
GO
-------------------------------------------------------------------------------
-- Insert query of interest here
-------------------------------------------------------------------------------
WITH CustomersOfInterest AS
(
	SELECT c.CustomerID, c.FirstName, c.LastName
	FROM CorpDB.dbo.Customer c
	WHERE c.State = 'NE'
)
SELECT oh.OrderId, oh.OrderDate, c.FirstName, c.LastName
FROM CorpDB.dbo.OrderHeader oh
	INNER JOIN CustomersOfInterest c ON oh.CustomerId = c.CustomerID
OPTION (RECOMPILE);
GO

SELECT * INTO #after_optimizer_info FROM sys.dm_exec_query_optimizer_info;
GO
SELECT a.counter, a.occurrence - b.occurrence occurrence, a.occurrence * a.value - b.occurrence * b.value value
FROM #before_optimizer_info b
	JOIN #after_optimizer_info a ON a.counter = b.counter
WHERE a.occurrence != b.occurrence ORDER BY counter;
GO
IF OBJECT_ID('tempdb..#before_optimizer_info') IS NOT NULL DROP TABLE #before_optimizer_info;
IF OBJECT_ID('tempdb..#after_optimizer_info') IS NOT NULL DROP TABLE #after_optimizer_info;
GO
-------------------------------------------------------------------------------
-- End
-------------------------------------------------------------------------------
